import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { EntityNotFoundError, IsNull, Repository } from 'typeorm';
import { ProductHistoryStatus } from '../entities/product-history-status.entity';

@Injectable()
export class ProductHistoryStatusService {
  constructor(
    @InjectRepository(ProductHistoryStatus)
    private productHistoryStatusRepository: Repository<ProductHistoryStatus>,
  ) {}

  async create(dataProduct: ProductHistoryStatus) {
    const result = await this.productHistoryStatusRepository.insert(
      dataProduct,
    );

    return result;
  }

  async findOneActive(productId: string, partnerId: string) {
    try {
      return await this.productHistoryStatusRepository.findOneOrFail({
        relations: ['product'],
        where: {
          product: productId,
          partner: partnerId,
        },
      });
    } catch (e) {
      if (e instanceof EntityNotFoundError) {
        throw new HttpException(
          {
            statusCode: HttpStatus.NOT_FOUND,
            error: 'Product not active',
          },
          HttpStatus.NOT_FOUND,
        );
      } else {
        throw e;
      }
    }
  }
}
