import { forwardRef, Module } from '@nestjs/common';
import { TransactionService } from './transaction.service';
import { TransactionController } from './transaction.controller';
import { PpobCallbackController } from './ppob_callback.controller';
import { TypeOrmModule } from '@nestjs/typeorm';
import { COA } from './entities/coa.entity';
import { TransactionJournal } from './entities/transaction-journal.entity';
import { Transactions } from './entities/transactions.entity';
import { CoaService } from './coa.service';
import { ProductModule } from '../product/product.module';
import { UsersModule } from 'src/users/users.module';
import { ConfigurableModule } from '../configurable/configurable.module';
import { CheckBillHistory } from './entities/check-bill-history.entity';
import { CallbackPartner } from './entities/callback-partner.entity';
import { ProductHistoryPrice } from '../product/entities/product-history-price.entity';
import { ProductHistoryStatus } from '../product/entities/product-history-status.entity';
import { ExcelController } from './excel.controller';

@Module({
  imports: [
    TypeOrmModule.forFeature([
      COA,
      TransactionJournal,
      Transactions,
      CheckBillHistory,
      CallbackPartner,
      ProductHistoryPrice,
      ProductHistoryStatus,
    ]),
    ProductModule,
    ConfigurableModule,
    forwardRef(() => {
      return UsersModule;
    }),
  ],
  controllers: [TransactionController, PpobCallbackController, ExcelController],
  providers: [TransactionService, CoaService],
  exports: [CoaService],
})
export class TransactionModule {}
